require 'test_helper'
require 'project_hierarchy_test_helper'

class ProjectExtensionTest < ActiveSupport::TestCase
  
  include ProjectHierarchyTestHelper

  test 'change parent' do
    parent_proj = Factory(:project, title: 'test parent')
    proj = Factory(:project, parent_id: parent_proj.id)
    assert_equal proj.parent, parent_proj
    assert parent_proj.descendants.include?(proj)
    parent_proj_changed = Factory(:project, title: 'changed test parent')
    proj.parent = parent_proj_changed
    disable_authorization_checks { proj.save! }

    assert_equal 'changed test parent', proj.parent.title
  end

  test 'create ancestor work groups after adding institutions' do
    institutions = [Factory(:institution), Factory(:institution)]
    parent_proj = Factory :project, title: 'parent proj'
    project = Factory :project, parent: parent_proj
    disable_authorization_checks { project.institutions = institutions }

    institutions.each do |ins|
      assert parent_proj.institutions.include?(ins)
    end
  end
  
  test 'related resource to parent project' do
    parent_proj = Factory :project
    proj = Factory :project, parent: parent_proj
    contributor = Factory(:person)
    contributor.add_to_project_and_institution(proj,Factory(:institution))

    User.with_current_user(contributor.user) do
      Project::RELATED_RESOURCE_TYPES.reject{|type| %w(Study Assay).include?(type)}.each do |type|
        item = Factory(type.underscore.to_sym)
        item.contributor = contributor if item.respond_to?(:contributor)
        assert proj.can_edit?
        proj.send "#{type.underscore.pluralize}=".to_sym, [item]

        proj.send("#{type.underscore.pluralize}".to_sym).each do |resource|
          assert parent_proj.send("related_#{type.underscore.pluralize}".to_sym).include?(resource)
        end
      end
    end
  end

  test 'projects with children cannot be deleted' do
    refute @proj.children.empty?
    refute @proj.can_delete?(Factory(:admin))
  end

end
